// Copyright 2019 RnD Center "ELVEES", JSC

#ifndef CVATPARSE_H_
#define CVATPARSE_H_

#include <vector>
#include <map>
#include <string>

namespace cvatparse {

struct point {
    float x;
    float y;
    point() {}
    point(float x_, float y_) : x(x_), y(y_) {}
};

struct base_object {
    std::string label;
    unsigned int z_order;
    std::map<std::string, std::string> attr;
    bool occluded;
    unsigned int frame;
    bool outside;
    bool keyframe;
    base_object() : occluded(false), z_order(0),
        outside(false), keyframe(true) {}
    // image meta constructors
    base_object(const std::string &_label,
                bool _occluded = false,
                unsigned int _z_order = 0)
                : label(_label), occluded(_occluded), z_order(_z_order) {}
    base_object(const std::string &_label,
                const std::map<std::string, std::string> &_attr,
                bool _occluded = false,
                unsigned int _z_order = 0)
                : label(_label), occluded(_occluded), z_order(_z_order),
                    attr(_attr) {}
    // track meta constructors
    base_object(unsigned int _frame,
                bool _occluded = false,
                bool _outside = false,
                bool _keyframe = false)
                : frame(_frame), occluded(_occluded), outside(_outside),
                    keyframe(_keyframe) {}
    base_object(unsigned int _frame,
                const std::map<std::string, std::string> &_attr,
                bool _occluded = false,
                bool _outside = false,
                bool _keyframe = false)
                : frame(_frame), occluded(_occluded), outside(_outside),
                    keyframe(_keyframe), attr(_attr) {}
};

struct box : base_object {
    point tl;
    point br;
    box() {}
    // image meta constructors
    box(const std::string &label,
        float xtl, float ytl, float xbr, float ybr,
        bool occluded = false,
        unsigned int z_order = 0)
        : base_object(label, occluded, z_order),
            tl({xtl, ytl}), br({xbr, ybr}) {}
    box(const std::string &label,
        float xtl, float ytl, float xbr, float ybr,
        const std::map<std::string, std::string> &attr,
        bool occluded = false,
        unsigned int z_order = 0)
        : base_object(label, attr, occluded, z_order),
            tl({xtl, ytl}), br({xbr, ybr}) {}
    // track meta constructors
    box(unsigned int frame,
        float xtl, float ytl, float xbr, float ybr,
        bool occluded = false,
        bool outside = false,
        bool keyframe = false)
        : base_object(frame, occluded, outside, keyframe),
            tl({xtl, ytl}), br({xbr, ybr}) {}
    box(unsigned int frame,
        float xtl, float ytl, float xbr, float ybr,
        const std::map<std::string, std::string> &attr,
        bool occluded = false,
        bool outside = false,
        bool keyframe = false)
        : base_object(frame, attr, occluded, outside, keyframe),
            tl({xtl, ytl}), br({xbr, ybr}) {}
};

struct points : base_object {
    std::vector<cvatparse::point> point;
    points() {}
    // image meta constructors
    points(const std::string &label,
           const std::vector<cvatparse::point> &_point,
           bool occluded = false,
           unsigned int z_order = 0)
           : base_object(label, occluded, z_order), point(_point) {}
    points(const std::string &label,
           const std::vector<cvatparse::point> &_point,
           const std::map<std::string, std::string> &attr,
           bool occluded = false,
           unsigned int z_order = 0)
           : base_object(label, attr, occluded, z_order), point(_point) {}
    // track meta constructors
    points(unsigned int frame,
           const std::vector<cvatparse::point> &_point,
           bool occluded = false,
           bool outside = false,
           bool keyframe = false)
           : base_object(frame, occluded, outside, keyframe), point(_point) {}
    points(unsigned int frame,
           const std::vector<cvatparse::point> &_point,
           const std::map<std::string, std::string> &attr,
           bool occluded = false,
           bool outside = false,
           bool keyframe = false)
           : base_object(frame, attr, occluded, outside, keyframe),
               point(_point) {}
};
typedef points polygon;
typedef points polyline;

struct image {
    unsigned int id;
    unsigned int height;
    unsigned int width;
    std::string name;
    std::vector<cvatparse::box> box;
    std::vector<cvatparse::points> points;
    std::vector<cvatparse::polygon> polygon;
    std::vector<cvatparse::polyline> polyline;
};

struct track {
    unsigned int id;
    std::string label;
    std::vector<cvatparse::box> box;
    std::vector<cvatparse::points> points;
    std::vector<cvatparse::polygon> polygon;
    std::vector<cvatparse::polyline> polyline;
};

struct annotations {
    std::vector<cvatparse::image> images;
    std::vector<cvatparse::track> tracks;
    std::map<std::string, std::string> alg_info;
};

bool load(const std::string &path, annotations &ann);
bool dump(const std::string &path, annotations &ann);

}  // namespace cvatparse

#endif  // CVATPARSE_H_
